<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/12/15
 * Time: 11:26 PM
 * To change this template use File | Settings | File Templates.
 */
class DSCF_DDL_DripLists_Item extends DSCF_DDL_StandardObjectRecord{

    public $titles = array();					// The title for the object.
    public $content = '';	                    // The description of additional text for the items
    public $featuredImages = array();		    // The WordPress media ID information for a featured image
    public $categories = array();               // The list of categories to use
    public $categoryUseAll = true;              // Whether or not to assign to all the categories or not

    public $dripListTopics=array();
    public $dripLastPostTimestamps=array();    // The timestamp of the last time this post was published

    const TABLE_NAME = "item";

    const DISPLAY_TYPE_FULL = 1;
    const DISPLAY_TYPE_TITLE_ONLY = 2;
    const DISPLAY_TYPE_TITLE_ONLY_LINKED = 3;
    const DISPLAY_TYPE_TITLE_IMAGE_ONLY = 4;
    const DISPLAY_TYPE_TITLE_IMAGE_ONLY_LINKED = 5;

    public function __construct(){

        global $wpdb;
        $table_name = self::getTableName();

        //Pass it to the parent to setup some common items
        $this->create();

        //Save in the DB
        try{
            $wpdb->insert(
                $table_name,
                array(
                    'blogId' => $this->blogId,
                    'data' => self::pack($this)
                ),
                array(
                    '%d',
                    '%s'
                )
            );
        }catch(Exception $e){
            //DSCF_DDL_Utilities::logMessage('Error creating Poll: '.$e->getMessage());
            return false;
        }

        //Get the ID of the inserted row and save it back to the object
        try{
            $this->id = $wpdb->insert_id;
            $this->save();
        }catch(Exception $e){
            //DSCF_DDL_Utilities::logMessage('Error creating Poll: '.$e->getMessage());
            return false;
        }

        //Return the object
        return $this;
    }

    public function save(){
        global $wpdb;
        $table_name = self::getTableName();
        $this->setLastModified();
        try{
            $wpdb->update(
                $table_name,
                array(
                    'data' => self::pack($this)
                ),
                array( 'ID' => $this->id ),
                array(
                    '%s'
                ),
                array( '%d' )
            );
        }catch(Exception $e){
            //DSCF_DDL_Utilities::logMessage('Error saving Object: '.$e->getMessage());
            return false;
        }
        return true;
    }

    // Controlled by Parent Class
    //public static function getTableName(){}

    // Controlled by Parent Class
    //public static function get($id){}

    // Controlled by Parent Class
    //public static function getAll($start=0,$limit=500){}

    // Controlled by Parent Class
    //public static function deleteById($id){}

    public static function getAllByTopicId($zoneId, $start=0, $limit=500){

        // Setup the variables
        global $wpdb;
        $tableName = self::getTableName();
        $zoneItemTableName = DSCF_DDL_ZoneItem::getTableName();

        // Query the Data
        $sql = $wpdb->prepare(
            "
            SELECT * FROM $tableName item
              INNER JOIN $zoneItemTableName zoneitem
              ON item.id = zoneitem.itemId
            WHERE zoneitem.zoneId = %d
            ORDER BY item.id DESC
            LIMIT %d,%d
            ",
            $zoneId,
            $start,
            $limit
        );

        // Get the results
        $packedObjects = self::executeQuery($sql, ARRAY_A);

        // Test the results are valid and unpack if so
        $goodObjects = self::unpackObjects($packedObjects);

        // Return the results
        return $goodObjects;

    }

    public static function editorBuildOptions($object){

        $topics = DSCF_DDL_DripLists_List::getAll();
        $topicsArray = DSCF_DDL_StandardObjectRecord::getBasicArrayFromObjects($topics, 'title');

        $options=array();
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Basic Content Item Properties',
        );
        $options[]=array(
            'title'=>'ID',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'id',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?$object->id:'',
            'help'=>''
        );
        $options[]=array(
            'title'=>'Titles',
            'objectType'=>DSCF_DDL_StandardEditor::OT_ARRAY,
            'objectName'=>'titles',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT_ADDITIONAL,
            'value'=>($object)?$object->titles:array(),
            'help'=>'Titles to use for this Content item.  Title will be randomly chosen.',
            'size'=>50,
        );
        $options[]=array(
            'title'=>'Content',
            'objectType'=>DSCF_DDL_StandardEditor::OT_STRING,
            'objectName'=>'content',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXTAREA_HTML,
            'value'=>($object)?$object->content:'',
            'help'=>'The Additional Content for this item.',
        );
        $options[]=array(
            'title'=>'Featured Images',
            'objectType'=>DSCF_DDL_StandardEditor::OT_ARRAY,
            'objectName'=>'featuredImages',
            'formType'=>DSCF_DDL_StandardEditor::ET_MEDIA_LIBRARY_MULTIPLE,
            'value'=>($object)?$object->featuredImages:array(),
            'help'=>'Featured Images to be use for this Content item.  Featured images will be randomly chosen. NOTE: This only applies if your current theme supports featured images.',
        );
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Drip List Topic Assignments',
        );
        $options[]=array(
            'title'=>'Assigned Drip List',
            'objectType'=>DSCF_DDL_StandardEditor::OT_ARRAY,
            'objectName'=>'dripListTopics',
            'formType'=>DSCF_DDL_StandardEditor::ET_ITEM_ZONE_PICKER,
            'value'=>($object)?DSCF_DDL_Zone::getZonesForItem($object->id):array(),
            'formOptions'=>$topicsArray,
            'help'=>'Select the Drip List Topics you want this item to be assigned to.',
        );
        /*$options[]=array(
            'title'=>'Last Post Timestamp',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'dailyDripLastPostTimestamp',
            'formType'=>DSCF_DDL_StandardEditor::ET_DATE_TIME,
            'value'=>($object)?$object->dailyDripLastPostTimestamp:'',
            'help'=>'The timestamp that this item was last published in the Daily Drip.',
        );*/
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Stats Tracking',
        );
        $options[]=array(
            'title'=>'Status',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'status',
            'formType'=>DSCF_DDL_StandardEditor::ET_MENU_STATUS,
            'formOptions'=>DSCF_DDL_StandardObjectRecord::getAllStatusMarks(),
            'value'=>($object)?$object->status:DSCF_DDL_StandardObjectRecord::STATUS_ACTIVE,
            'help'=>'Select whether this Item is active or inactive'
        );
        $options[]=array(
            'title'=>'Created Date',
            'objectType'=>DSCF_DDL_StandardEditor::OT_DATE,
            'objectName'=>'createdDate',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?DSCF_DDL_StandardObjectRecord::formatDate($object->createdDate):'',
            'help'=>'The date the Item was created on'
        );
        return $options;
    }

    public static function managerBuildOptions($objects){
        $rows = array();
        foreach($objects as $object){
            $row = array();
            $row[] = array(
                'title'=>'ID',
                'content'=>$object->id,
            );
            $row[] = array(
                'title'=>'Status',
                'content'=>$object->getFormattedStatusImage($object->status, '24px'),
            );
            $row[] = array(
                'title'=>'Edit',
                'url'=>DSCF_DDL_Utilities::getPageUrl(DSCF_DDL_DripLists_Main::PAGE_ITEMS, array('id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DDL_URL_IMAGES.'/page_edit.png',
                'image_tooltip'=>'Edit Top Ten Item',
            );
            $title = "";
            if(DSCF_DDL_Utilities::validateArray($object->titles)){
                foreach($object->titles as $title){
                    if(!empty($title)){
                        break;
                    }
                }
            }
            $row[] = array(
                'title'=>'First Title',
                'content'=>$title,
            );
            $src = "";
            if(DSCF_DDL_Utilities::validateArray($object->featuredImages)){
                foreach($object->featuredImages as $featuredImage){
                    if(!empty($featuredImage)){
                        $src = wp_get_attachment_thumb_url($featuredImage);
                        break;
                    }
                }
            }
            $row[] = array(
                'title'=>'Example Image',
                'content'=>(!empty($src))?'<img src="'.$src.'" width="50" height="50" style="max-height: 50px; width: 50px;" />':'',
            );
            $row[] = array(
                'title'=>'Counts',
                'multiple'=>true,
                'rows'=>array(
                    0=>array(
                        'subTitle'=>'Titles',
                        'content'=>count($object->titles),
                    ),
                    1=>array(
                        'subTitle'=>'Images',
                        'content'=>count($object->featuredImages),
                    ),
                ),
            );
            $row[] = array(
                'title'=>'Created Date',
                'content'=>DSCF_DDL_StandardObjectRecord::formatSiteDate($object->createdDate),
            );
            $row[] = array(
                'title'=>'Delete',
                'url'=>DSCF_DDL_Utilities::getPageUrl(DSCF_DDL_DripLists_Main::PAGE_ITEMS, array('delete'=>1,'id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DDL_URL_IMAGES.'/delete.png',
                'image_tooltip'=>'Delete Top Ten item',
            );
            $rows[]=$row;
        }
        return $rows;
    }

    // Function to create a wordpress post from the contents for this advice item
    public function createWordPressPost($listId=0, $timestamp=null){

        // Setup the basics
        $params = array(
            'id'=>0,    // Making sure this is 0 so a new post gets created
            'post_content'=>$this->content,
            'post_title'=>'',
            'post_author'=>0,
        );

        // Set the Title
        $title = DSCF_DDL_Utilities::getRandomArrayEntry($this->titles);
        if(!empty($title)){
            $params['post_title']=$title;
        }

        DSCF_DDL_Utilities::logMessage("Timestamp to use for Post: ".$timestamp);

        // Check to see if we need to change the status based on the timestamp
        if(!empty($timestamp)){
            // A timestamp was passed
            $params['post_date_gmt'] = date("Y-m-d H:i:s", $timestamp);
            $currentTime = time();
            if($timestamp > $currentTime){
                // To be scheduled in the future
                $params['post_status'] = 'future';
            }else{
                // Scheduled in the past
                $params['post_status'] = 'publish';
            }
        }

        $postId = wp_insert_post($params);

        // Set the thumbnail
        $attachId = DSCF_DDL_Utilities::getRandomArrayEntry($this->featuredImages);
        if(!empty($attachId)){
            set_post_thumbnail( $postId, $attachId );
        }

        $this->dripLastPostTimestamps[$listId] = (empty($timestamp))?time():$timestamp;
        $this->save();

        return $postId;

    }



}
