<?php
/**
 * Created by JetBrains PhpStorm.
 * User: ben
 * Date: 3/12/15
 * Time: 11:26 PM
 * To change this template use File | Settings | File Templates.
 */
class DSCF_DDL_DripLists_List extends DSCF_DDL_Zone{

    public $title;					    // The title for the object.
    public $tags=array();

    public $lastPostTimestamp=0;            // The timestamp of the last automatic post - this value may be in the future
    public $lastPostId=0;                   // The WordPress Post ID for the last post that was made
    public $lastItemId=0;                   // The last Item ID that was posted

    public $nextPostTimestamp=0;            // The timestamp to use for the next automatic post

    public $autoPostingCooldown=64800;      // The cooldown the next post timestamp must be within to schedule a post.  Cron will check twice daily

    public $scheduleDaysOfTheWeek=array();          // The days of the week to publish an article on
    public $scheduleCooldownDays=0;                 // The cooldown days between article publishes
    public $scheduleTimeOfDay=0;                    // Approximate time of day to perform the article post
    public $scheduleRandomizeStartTimeMinutes=0;    // The number of minutes to randomize the post start time

    public function __construct(){
        $this->typeId = self::TYPE_DDL;
        parent::__construct();
        $this->ensureValidDates();
    }

    public static function getAll($start=0, $limit=500){
        return self::getAllByTypeId(self::TYPE_DDL, $start, $limit);
    }

    public static function editorBuildOptions($object){

        $options=array();
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Basic List Properties',
        );
        $options[]=array(
            'title'=>'ID',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'id',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?$object->id:'',
            'help'=>''
        );
        $options[]=array(
            'title'=>'Title',
            'objectType'=>DSCF_DDL_StandardEditor::OT_STRING,
            'objectName'=>'title',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT,
            'value'=>($object)?$object->title:'',
            'help'=>'The title for this List.',
            'size'=>100,
        );
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Schedule Settings',
        );
        // How often to schedule content
        $instructions = '';
        $instructions .= '<p>Choose settings to control which days of the week articles are published on. <button onclick="jQuery(\'#dimbalSettingsScheduleInstructions\').toggle();return false;">Toggle Help</button></p>';
        $instructions .= '<div id="dimbalSettingsScheduleInstructions" style="display:none;">';
        $instructions .= '<p>Ex 1: To have your articles published once a week on Mondays, choose "Monday" for the Eligible Days of the Week and "0" for the cooldown time period.</p>';
        $instructions .= '<p>Ex 2: To have your articles published every 2 weeks on Sundays, choose "Sunday" for the Eligible Days of the Week and "1" for the cooldown time period.</p>';
        $instructions .= '<p>Ex 3: To have your articles published every Monday, Wednesday and Friday, choose "Monday", "Wednesday" and "Friday" for the Eligible Days of the Week and "0" for the cooldown time period.</p>';
        $instructions .= '<p>Ex 4: To have your articles published every 3 days, but only on weekdays, choose "Monday", "Tuesday", "Wednesday", "Thursday" and "Friday" for the Eligible Days of the Week and "2" for the cooldown time period.</p>';
        $instructions .= '</div>';
        $options[]=array(
            'title'=>'Settings Explanation',
            'objectType'=>DSCF_DDL_StandardEditor::OT_SKIP,
            'objectName'=>'skip',
            'formType'=>DSCF_DDL_StandardEditor::ET_HTML,
            'value'=>$instructions,
        );
        $options[]=array(
            'title'=>'Eligible Days of the Week',
            'objectType'=>DSCF_DDL_StandardEditor::OT_ARRAY,
            'objectName'=>'scheduleDaysOfTheWeek',
            'formType'=>DSCF_DDL_StandardEditor::ET_CHECKBOX_GROUP,
            'formOptions'=>DSCF_DDL_Utilities::getDaysOfWeek(),
            'value'=>($object)?$object->scheduleDaysOfTheWeek:array(0,1,2,3,4,5,6),
            'help'=>'Choose the days of the week that are eligible for publishing new articles on.',
        );
        $options[]=array(
            'title'=>'Cooldown Days',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'scheduleCooldownDays',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT,
            'value'=>($object)?$object->scheduleCooldownDays:0,
            'help'=>'How many eligible days should elapse between new article publishes.  Enter 0 to publish a new article every eligible publishing day.  Enter 1 to publish every other eligible day, etc.',
            'size'=>10,
        );
        $instructions = '';
        $instructions .= '<p>Choose settings to control what time of day articles should be published on. <button onclick="jQuery(\'#dimbalSettingsTimeframeInstructions\').toggle();return false;">Toggle Help</button></p>';
        $instructions .= '<div id="dimbalSettingsTimeframeInstructions" style="display:none;">';
        $instructions .= '<p>Ex 1: To have your articles published at 9am, choose "09:00AM" for the Time of day and "0" minutes for randomization.</p>';
        $instructions .= '<p>Ex 2: To have your articles published randomly between 1pm and 3pm, choose "02:00PM" for the Time of day and "60" minutes for randomization before and after the start time.</p>';
        $instructions .= '</div>';
        $options[]=array(
            'title'=>'Timeframe Settings',
            'objectType'=>DSCF_DDL_StandardEditor::OT_SKIP,
            'objectName'=>'skip',
            'formType'=>DSCF_DDL_StandardEditor::ET_HTML,
            'value'=>$instructions,
        );
        $options[]=array(
            'title'=>'Approximate Time of Day',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'scheduleTimeOfDay',
            'formType'=>DSCF_DDL_StandardEditor::ET_MENU_KEY,
            'formOptions'=>DSCF_DDL_Utilities::getTimeArray(),
            'value'=>($object)?$object->scheduleTimeOfDay:720,
            'help'=>'Choose the start time for when an article can be published.',
        );
        $options[]=array(
            'title'=>'Randomize Start Time Minutes',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'scheduleRandomizeStartTimeMinutes',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT,
            'value'=>($object)?$object->scheduleRandomizeStartTimeMinutes:0,
            'help'=>'If desired, how many minutes to randomize the schedule by (before and after) from the desired publish timeframe.',
            'size'=>10,
        );
        $options[]=array(
            'rowType'=>'SectionHeader',
            'title'=>'Other Settings',
        );
        $options[]=array(
            'title'=>'Status',
            'objectType'=>DSCF_DDL_StandardEditor::OT_NUMERIC,
            'objectName'=>'status',
            'formType'=>DSCF_DDL_StandardEditor::ET_MENU_STATUS,
            'formOptions'=>DSCF_DDL_StandardObjectRecord::getAllStatusMarks(),
            'value'=>($object)?$object->status:DSCF_DDL_StandardObjectRecord::STATUS_ACTIVE,
            'help'=>'Select whether this List is active or inactive'
        );
        $options[]=array(
            'title'=>'Created Date',
            'objectType'=>DSCF_DDL_StandardEditor::OT_SKIP,
            'objectName'=>'createdDate',
            'formType'=>DSCF_DDL_StandardEditor::ET_TEXT_READONLY,
            'value'=>($object)?DSCF_DDL_StandardObjectRecord::formatDate($object->createdDate):'',
            'help'=>'The date the List was created on'
        );
        return $options;
    }

    public static function managerBuildOptions($objects){
        $rows = array();
        foreach($objects as $object){

            $object->ensureValidDates(true);

            $row = array();
            $row[] = array(
                'title'=>'ID',
                'content'=>$object->id,
            );
            $row[] = array(
                'title'=>'Status',
                'content'=>$object->getFormattedStatusImage($object->status, '24px'),
            );
            $row[] = array(
                'title'=>'Edit',
                'url'=>DSCF_DDL_Utilities::getPageUrl(DSCF_DDL_DripLists_Main::PAGE_LISTS, array('id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DDL_URL_IMAGES.'/page_edit.png',
                'image_tooltip'=>'Edit List',
            );
            $row[] = array(
                'title'=>'Title',
                'content'=>$object->title,
            );

            $row[] = array(
                'title'=>'Next Post (Estimated)',
                'content'=>DSCF_DDL_StandardObjectRecord::formatSiteDate($object->nextPostTimestamp),
            );

            $row[] = array(
                'title'=>'Post Content Now',
                'url'=>DSCF_DDL_Utilities::getPageUrl(DSCF_DDL_DripLists_Main::PAGE_LISTS, array('publishPost'=>1,'id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DDL_URL_IMAGES.'/page_add.png',
                'image_tooltip'=>'Post Content Now',
            );

            $lastPost = "n/a";
            if(!empty($object->lastPostId)){
                $lastPost = '<a href="'.get_edit_post_link($object->lastPostId).'">'.$object->lastPostId.'</a>';
            }
            $row[] = array(
                'title'=>'Last Post',
                'content'=>$lastPost,
            );

            $lastPostTimestamp = "n/a";
            if(!empty($object->lastPostTimestamp)){
                $lastPostTimestamp = DSCF_DDL_StandardObjectRecord::formatSiteDate($object->lastPostTimestamp);
            }
            $row[] = array(
                'title'=>'Last Post Timestamp',
                'content'=>$lastPostTimestamp,
            );



            /*$row[] = array(
                'title'=>'Created Date',
                'content'=>DSCF_DDL_StandardObjectRecord::formatSiteDate($object->createdDate),
            );*/
            $row[] = array(
                'title'=>'Delete',
                'url'=>DSCF_DDL_Utilities::getPageUrl(DSCF_DDL_DripLists_Main::PAGE_LISTS, array('delete'=>1,'id'=>$object->id)),
                'image'=>WPDEVHUB_CONST_DDL_URL_IMAGES.'/delete.png',
                'image_tooltip'=>'Delete Topic',
            );
            $rows[]=$row;
        }
        return $rows;
    }

    /*
     * Simple function to ensure the dates we need to do calculations off of are healthy
     */
    public function ensureValidDates($save=false){
        if(empty($this->nextPostTimestamp)){
            // Need to set up the next timestamp
            $lastPostTimestamp = $this->lastPostTimestamp;
            if(empty($lastPostTimestamp)){
                $lastPostTimestamp = time();
            }
            $this->nextPostTimestamp = $this->getPublishTimestampAfterTimestamp($lastPostTimestamp);
            if($save){
                $this->save();
            }
        }
    }

    /*
     * Get the next Timestamp to publish on based on a passed timestamp.
     * Does not update any settings -- that way this can be used for reports and such "Approx next publish at... etc"
     */
    public function getPublishTimestampAfterTimestamp($currentTimestamp){

        // Need to first equalize according to Site Settings
        //$currentTimestamp = DSCF_DDL_StandardObjectRecord::formatTimestampForSite($currentTimestamp);

        // Figure out which day of week we are
        $currentDay = date("w", $currentTimestamp);

        // Get the cooldown from the settings
        $cooldownDays = $this->scheduleCooldownDays;
        if(empty($cooldownDays)){
            $cooldownDays = 0;
        }

        // Get the approved days from the settings
        $approvedDays = $this->scheduleDaysOfTheWeek;
        $approvedDaysKeys = array();
        if(empty($approvedDays) || !is_array($approvedDays)){
            // Log an error and get out of here

        }else{
            $approvedDaysKeys = array_flip($approvedDays);
        }

        $searchForDate = true;
        $secondsToAdd = 0;
        $runawayPrevention = 0;
        $newTimestamp = 0;

        while($searchForDate){
            $runawayPrevention++;
            if($runawayPrevention > 10){
                // An error is occuring - maybe bad data with the array, etc..
                // LOG AN ERROR
                return false;
            }
            $secondsToAdd += 86400;     // Add a days worth of time
            $newTimestamp = $currentTimestamp + $secondsToAdd;
            $dayOfWeek = date("w", $newTimestamp);
            if(is_array($approvedDaysKeys) && array_key_exists($dayOfWeek, $approvedDaysKeys)){
                // This date is eligible for publishing
                if(empty($cooldownDays)){
                    // WE HAVE FOUND OUR DAY!!!
                    $searchForDate = false;
                }else{
                    $cooldownDays--;
                }
                $runawayPrevention = 0;
            }
        }

        // At this point we should have a timestamp
        if(empty($newTimestamp)){
            return false;
        }

        // Get the set minutes for the day
        $targetMinutes = 0;
        if(!empty($this->scheduleTimeOfDay)){
            $targetMinutes = $this->scheduleTimeOfDay;
        }

        // Get the mins to randomize by
        $minsToRandomize=0;
        if(!empty($this->scheduleRandomizeStartTimeMinutes)){
            $minsToRandomize = $this->scheduleRandomizeStartTimeMinutes;
        }

        // Now get the randomized number of mins to use
        $startMinutes = max(2, ($targetMinutes - $minsToRandomize));    // Plus 2 mins for safety padding
        $endMinutes = min(1438, ($targetMinutes + $minsToRandomize));   // 1440 minutes in a given day (minus 2 mins for safety padding)
        $minutesToUse = rand($startMinutes, $endMinutes);

        // Equalize the date according to the time of day
        $newTimestamp = mktime(0,$minutesToUse,0,date('m', $newTimestamp), date('j', $newTimestamp), date('Y', $newTimestamp));

        // Be sure to equalize according to Site Settings
        $newTimestamp = DSCF_DDL_StandardObjectRecord::formatSiteTimestampToGmtTimestamp($newTimestamp);

        // Now return it
        return $newTimestamp;

    }


    // Checks to see if this list needs to do an automated post
    public function checkForAutomatedPost($force = false){

        $postId = null;
        $time = time();
        $nextTimestamp = $this->nextPostTimestamp;
        $lastTimestamp = $this->lastPostTimestamp;
        $cooldown = $this->autoPostingCooldown;

        if($force || empty($nextTimestamp) || (($nextTimestamp - $cooldown) < $time)){
            // We are close enough... Schedule the post

            // If the last timestamp was empty (generally means new installs) temp set it to now - 1;
            if(empty($lastTimestamp)){
                $lastTimestamp = time()-1;
            }

            // Some validations on the dates
            if($force){
                // If being forced - then ignore other settings and use now as the timestamp
                $nextTimestamp = time();
            }elseif(empty($nextTimestamp)){
                // Next is empty - but last is not - get next accordingly
                $nextTimestamp = self::getPublishTimestampAfterTimestamp($lastTimestamp);
            }elseif($nextTimestamp < (time()-31536000)){
                // if the value got corrupted and reset to 1960, etc...
                $nextTimestamp = self::getPublishTimestampAfterTimestamp($lastTimestamp);
            }

            // Get the time to post
            $allItems = DSCF_DDL_DripLists_Item::getAllByTopicId($this->id);
            //shuffle($allItems);
            $oldestItem = null;
            $oldestItemTimestamp = time();
            if(DSCF_DDL_Utilities::validateArray($allItems)){
                foreach($allItems as $item){
                    //DSCF_DDL_Utilities::logMessage("Last Timestamps: ListId[".$this->id."] ItemID[".$item->id."]:  ".print_r($item->dripLastPostTimestamps, true));
                    // Get the last timestamp of this item for this topic.  Default to 0 if empty (means hasn't posted yet)
                    $itemTimestamp = DSCF_DDL_Utilities::getFromArray($this->id, $item->dripLastPostTimestamps, 0);
                    //DSCF_DDL_Utilities::logMessage("Item Timestamp: ListId[".$this->id."] ItemID[".$item->id."]:  ".$itemTimestamp);
                    if(empty($oldestItem) || ($itemTimestamp < $oldestItemTimestamp)){
                        DSCF_DDL_Utilities::logMessage("Item Timestamp was empty or less then oldest: ListId[".$this->id."] ItemID[".$item->id."]:  OldestTimestamp[$oldestItemTimestamp] itemTimestamp[$itemTimestamp]");
                        $oldestItem = $item;
                        $oldestItemTimestamp = $itemTimestamp;
                    }
                }
            }

            $postId = 0;
            $item = $oldestItem;
            if(!empty($item)){
                $postId = $item->createWordPressPost($this->id, $nextTimestamp);

                if(!empty($postId)){
                    $this->lastPostTimestamp = $nextTimestamp;
                    $this->nextPostTimestamp = $this->getPublishTimestampAfterTimestamp($nextTimestamp);
                    $this->lastPostId = $postId;
                    $this->lastItemId = $item->id;
                    $this->save();
                }

            }

        }

        return $postId;

    }

}
